local HouseAddition = ...

function registerAssignableFunction(_assetId)
    HouseAddition:register({
		DataType = "BUILDING_FUNCTION_LIST",
                Id = _HomePrefix .. "_" .. _assetId .. "_FUNCTION_LIST",
                AssetFunctionList = {
                        registerFunctionHouse(_HomePrefix, _assetId, "1"),
                        registerFunctionHouse(_HomePrefix, _assetId, "2")
                }
    })
    HouseAddition:register({
		DataType = "BUILDING_FUNCTION_ASSIGNABLE",
                Id = _HomePrefix .. "_" .. _assetId .. "_ASSIGNABLE_FUNCTION",
                AuthorizedFunctionList = _HomePrefix .. "_" .. _assetId .. "_FUNCTION_LIST"
    })
    return _HomePrefix .. "_" .. _assetId .. "_ASSIGNABLE_FUNCTION"
end

function registerHouseBuilding(_fbxName, _buildingAssetId, _buildingZone, _villagerCapacity)
-- Apply building asset processor
    HouseAddition:registerAssetProcessor("models/" .. _fbxName .. ".fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
    })

-- Register common prefab nodes
    HouseAddition:registerAssetId("models/" .. _fbxName .. ".fbx/Prefab/homePart", _HomePrefix .. "_" .. _buildingAssetId .. "_PREFAB")

-- Rgister buildingPart for random placement
    HouseAddition:register({
	DataType = "BUILDING_PART",
	Id = _HomePrefix .. "_" .. _buildingAssetId .. "_PART",
	Name = _HomePrefix .. "_" .. _buildingAssetId .. "_NAME",
	Description = _HomePrefix .. "_" .. _buildingAssetId .. "_DESC",
	Category = "CORE",
	BuildingZone = _buildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = _HomePrefix .. "_" .. _buildingAssetId .. "_PREFAB"
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = {
			{ Resource = "WOOD", Quantity = 0 }
		}
	}
    })

-- Register buildingPart for manual placement
    HouseAddition:register({
	DataType = "BUILDING_PART",
	Id = _HomePrefix .. "_PLACEABLE_" .. _buildingAssetId .. "_PART",
	Name = _HomePrefix .. "_" .. _buildingAssetId .. "_NAME",
	Description = _HomePrefix .. "_" .. _buildingAssetId .. "_DESC",
	Category = "CORE",
	BuildingZone = _buildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = _HomePrefix .. "_" .. _buildingAssetId .. "_PREFAB"
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = _NoSandbox*0 }
		},
		RessourcesNeeded = {
			{ Resource = "WOOD", Quantity = _NoSandbox*10 }
		}
	},
        VillagerCapacity = _villagerCapacity,
	AssetBuildingFunction = registerAssignableFunction(_buildingAssetId)
    })

-- Building grounded
    HouseAddition:registerPrefabComponent("models/" .. _fbxName .. ".fbx/Prefab/homePart", {
	DataType = "COMP_GROUNDED"
    })
    HouseAddition:registerPrefabComponent("models/" .. _fbxName .. ".fbx/Prefab/homePart/step_0/porte", {
	DataType = "COMP_GROUNDED"
    })

-- The fireplace...
    HouseAddition:registerAssetId("models/" .. _fbxName .. ".fbx/Prefab/homePart/step_2/cheminee/SmokeEmitter", _HomePrefix .. "_" .._buildingAssetId .. "_SMOKE_EMITTER_PREFAB")

    HouseAddition:registerPrefabComponent("models/" .. _fbxName .. ".fbx/Prefab/homePart", {
	DataType = "COMP_BUILDING_PART",
	FeedbackComponentListToActivate = {
                 { _HomePrefix .. "_" .. _buildingAssetId .. "_SMOKE_EMITTER_PREFAB", "COMP_PARTICLE_EMITTER_TOGGLE" }
	}
    })

    HouseAddition:registerPrefabComponent("models/" .. _fbxName .. ".fbx/Prefab/homePart/step_2/cheminee/SmokeEmitter", {
	DataType = "COMP_PARTICLE_EMITTER",
	ParticleSystem = "PARTICLE_SYSTEM_HOUSING_SMOKE",
	IsPlaying = true,
	IsEmitting = true
    })

    HouseAddition:registerPrefabComponent("models/" .. _fbxName .. ".fbx/Prefab/homePart/step_2/cheminee/SmokeEmitter", {
	DataType = "COMP_PARTICLE_EMITTER_TOGGLE",
	Enabled = false
    })
end

registerHouseBuilding("Home2", "HOME2",
                      { ZoneEntryList = {
                          { Polygon = polygon.createRectangle( {3, 7} ),
                            Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } }
                        }
                      }, 4)
registerHouseBuilding("Home3", "HOME3",
                      { ZoneEntryList = {
                          { Polygon = polygon.createRectangle( {4, 8}, {2, 0} ),
                            Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } },
                          { Polygon = polygon.createRectangle( {8, 4}, {0, 2} ),
                            Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } }
                        }
                      }, 6)
registerHouseBuilding("Home4", "HOME4",
                      { ZoneEntryList = {
                          { Polygon = polygon.createRectangle( {4, 8} ),
                            Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } }
                        }
                      }, 6)
registerHouseBuilding("Home5", "HOME5",
                      { ZoneEntryList = {
                          { Polygon = polygon.createRectangle( {4, 6} ),
                            Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } }
                        }
                      }, 6)
registerHouseBuilding("Home6", "HOME6",
                      { ZoneEntryList = {
                          { Polygon = polygon.createRectangle( {4, 7} ),
                            Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } }
                        }
                      }, 6)

--------------------------------------------------------------------------
function registerBuildingAssemblage(_fbxName, _buildingAssetId, _buildingZone, _villagerCapacity)
-- Apply building asset processor
    HouseAddition:registerAssetProcessor("models/" .. _fbxName .. ".fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
    })
 
-- Register prefab nodes
    HouseAddition:registerAssetId("models/" .. _fbxName .. ".fbx/Prefab/assemblyHouse/homePart", _HomePrefix .. "_" .. _buildingAssetId .. "_PREFAB")

-- Register the assemblage part for random part
    HouseAddition:register({
	DataType = "BUILDING_PART",
	Id = _HomePrefix .. "_" .. _buildingAssetId .. "_PART",
	Name = _HomePrefix .. "_" .. _buildingAssetId .. "_NAME",
	Description = _HomePrefix .. "_" .. _buildingAssetId .. "_DESC",
	Category = "CORE",
	BuildingZone = _buildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_ASSEMBLAGE",
		CoreRandomBuildingPartList = { _HomePrefix .. "_" .. _buildingAssetId .. "_PREFAB" },
                MandatoryBuildingPartList = {
                    { BuildingPart = _HomePrefix .. "_WOODEN_DOOR_PART" }
                    ,{ BuildingPart = _HomePrefix .. "_FENETRE_PART", Probability = 0.5 }
                    ,{ BuildingPart = _HomePrefix .. "_FENETRE_PART", Probability = 0.5 }
                    ,{ BuildingPart = _HomePrefix .. "_FENETRE_PART", Probability = 0.5 }
                    ,{ BuildingPart = _HomePrefix .. "_FENETRE_PART", Probability = 0.5 }
                    ,{ BuildingPart = _HomePrefix .. "_FENETRE_PART", Probability = 0.5 }
               }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "WOOD", Quantity = 0 }
		}
	}
    })

-- Register the assemblage part for manual part
    HouseAddition:register({
	DataType = "BUILDING_PART",
	Id = _HomePrefix .. "_PLACEABLE_" .. _buildingAssetId .. "_PART",
	Name = _HomePrefix .. "_" .. _buildingAssetId .. "_NAME",
	Description = _HomePrefix .. "_" .. _buildingAssetId .. "_DESC",
	Category = "CORE",
	BuildingZone = _buildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_ASSEMBLAGE",
		CoreRandomBuildingPartList = { _HomePrefix .. "_" .. _buildingAssetId .. "_PREFAB" },
                MandatoryBuildingPartList = {
                    { BuildingPart = _HomePrefix .. "_WOODEN_DOOR_PART" }
                    ,{ BuildingPart = _HomePrefix .. "_FENETRE_PART", Probability = 0.5 }
                    ,{ BuildingPart = _HomePrefix .. "_FENETRE_PART", Probability = 0.5 }
                    ,{ BuildingPart = _HomePrefix .. "_FENETRE_PART", Probability = 0.5 }
                    ,{ BuildingPart = _HomePrefix .. "_FENETRE_PART", Probability = 0.5 }
                    ,{ BuildingPart = _HomePrefix .. "_FENETRE_PART", Probability = 0.5 }
               }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = _NoSandbox*0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "WOOD", Quantity = _NoSandbox*10 }
		}
	},
        VillagerCapacity = _villagerCapacity,
	AssetBuildingFunction = registerAssignableFunction(_buildingAssetId)
    })

    HouseAddition:registerPrefabComponent("models/" .. _fbxName .. ".fbx/Prefab/assemblyHouse/homePart", {
	DataType = "COMP_GROUNDED"
    })

-- several models of door
    HouseAddition:register({
	DataType = "BUILDING_PART",
	Id = _HomePrefix .. "_WOODEN_DOOR_PART",
	ConstructorData = {
		          DataType = "BUILDING_CONSTRUCTOR_RANDOM_PART",
		          RandomPartList = {
                                _HomePrefix .. "_DOOR1_PART"
                                , _HomePrefix .. "_DOOR2_PART"
                                , _HomePrefix .. "_DOOR3_PART"
                          }
        }
    })

    -- Register the door parts
    function registerPortePart(_doorName, _doorAssetId, _doorZone)
        HouseAddition:registerAssetId("models/" .. _fbxName .. ".fbx/Prefab/assemblyHouse/" .. _doorName .. "Part", _HomePrefix .. _doorAssetId .. "_PREFAB")

        HouseAddition:register({
        	DataType = "BUILDING_PART",
	        Id = _HomePrefix .. _doorAssetId .. "_PART",
	        ConstructorData = {
		    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		    CoreObjectPrefab = _HomePrefix .. _doorAssetId .. "_PREFAB"
	        }
        })

        HouseAddition:registerPrefabComponent("models/" .. _fbxName .. ".fbx/Prefab/assemblyHouse/" .. _doorName .. "Part", {
	    DataType = "COMP_BUILDING_PART",
            BuildingPartType = "MAJOR"
        })

        HouseAddition:registerPrefabComponent("models/" .. _fbxName .. ".fbx/Prefab/assemblyHouse/" .. _doorName .. "Part", {
	    DataType = "COMP_GROUNDED"
        })
    end

    registerPortePart("door1", "_DOOR1", { 0.1, 1 })
    registerPortePart("door2", "_DOOR2", { 0.1, 1 })
    registerPortePart("door3", "_DOOR3", { 1, 1 })

-- Register the fenetre part
    HouseAddition:registerAssetId("models/" .. _fbxName .. ".fbx/Prefab/assemblyHouse/fenetrePart", _HomePrefix .. "_FENETRE_PREFAB")

    HouseAddition:register({
	DataType = "BUILDING_PART",
	Id = _HomePrefix .. "_FENETRE_PART",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = _HomePrefix .. "_FENETRE_PREFAB"
	}
    })

    HouseAddition:registerPrefabComponent("models/" .. _fbxName .. ".fbx/Prefab/assemblyHouse/fenetrePart", {
	DataType = "COMP_BUILDING_PART",
        BuildingPartType = "MAJOR",
    })

-- The fireplace...
    HouseAddition:registerAssetId("models/" .. _fbxName .. ".fbx/Prefab/assemblyHouse/homePart/step_2/cheminee/SmokeEmitter", _HomePrefix .. "_" .. _buildingAssetId .. "_SMOKE_EMITTER_PREFAB")

    HouseAddition:registerPrefabComponent("models/" .. _fbxName .. ".fbx/Prefab/assemblyHouse/homePart", {
	DataType = "COMP_BUILDING_PART",
	FeedbackComponentListToActivate = {
                 { _HomePrefix .. "_" .. _buildingAssetId .. "_SMOKE_EMITTER_PREFAB", "COMP_PARTICLE_EMITTER_TOGGLE" }
	}
    })

    HouseAddition:registerPrefabComponent("models/" .. _fbxName .. ".fbx/Prefab/assemblyHouse/homePart/step_2/cheminee/SmokeEmitter", {
	DataType = "COMP_PARTICLE_EMITTER",
	ParticleSystem = "PARTICLE_SYSTEM_HOUSING_SMOKE",
	IsPlaying = true,
	IsEmitting = true
    })

    HouseAddition:registerPrefabComponent("models/" .. _fbxName .. ".fbx/Prefab/assemblyHouse/homePart/step_2/cheminee/SmokeEmitter", {
	DataType = "COMP_PARTICLE_EMITTER_TOGGLE",
	Enabled = false
    })
end

registerBuildingAssemblage("Home11", "HOME11",
                      { ZoneEntryList = {
                          { Polygon = polygon.createRectangle( {4, 8} ),
                            Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } }
                        }
                      }, 6)


